import fs from 'node:fs'
import path from 'node:path'
import { fileURLToPath } from 'node:url'
import esbuild from 'esbuild'

const __dirname = path.dirname(fileURLToPath(import.meta.url))
const VENDOR_DIR = path.join(__dirname, '../vendor/shiki')

function createDeclarationFile(outputPath, name) {
  // Only create declarations for themes and langs
  if (!outputPath.includes('/themes/') && !outputPath.includes('/langs/')) {
    return
  }

  // Remove .js extension and create paths
  const jsPath = outputPath.replace(/\\/g, '/')
  const dtsPath = jsPath.replace('.js', '.d.ts')
  const modulePath = jsPath.replace('.js', '')

  // Get the correct type based on the path
  const typeAnnotation = outputPath.includes('/themes/') ? 'import("shiki").ThemeInput' : 'import("shiki").LanguageInput'

  // Replace hyphens in name to create a valid TypeScript identifier
  const safeName = name.replace(/-/g, '_')

  const dtsContent = `\
import type { ThemeInput, LanguageInput } from 'shiki'

declare module '${modulePath}' {
  const ${safeName}: ${typeAnnotation}
  export default ${safeName}
}
`
  fs.writeFileSync(dtsPath, dtsContent)
}

// Build function
async function buildModule(inputPath, outputPath, typeName = 'any') {
  try {
    const result = await esbuild.build({
      entryPoints: [inputPath],
      bundle: true,
      write: false,
      format: 'cjs',
      platform: 'node',
      external: ['react', 'react-native'],
      define: {
        'process.env.NODE_ENV': '"production"',
      },
    })

    if (inputPath.includes('/themes/')) {
      const themeContent = result.outputFiles[0].text
      const match = themeContent.match(/JSON\.parse\('(.+?)'\)/)
      if (match) {
        fs.writeFileSync(outputPath, `module.exports = ${match[1]}`)
      }
      else {
        throw new Error('Could not find theme data in bundle')
      }
    }
    else {
      fs.writeFileSync(outputPath, result.outputFiles[0].text)
    }

    // Create declaration file for TypeScript, replacing hyphens in name
    createDeclarationFile(outputPath, path.basename(outputPath, '.js').replace(/-/g, '_'), typeName)
    return true
  }
  catch (error) {
    console.error(`Failed to build ${inputPath}:`, error)
    return false
  }
}

// Create initial manifest with metadata
function createManifest() {
  const shikiPkg = JSON.parse(fs.readFileSync(path.join(__dirname, '../node_modules/shiki/package.json'), 'utf8'))

  return {
    version: shikiPkg.version,
    buildDate: new Date().toISOString(),
    core: [],
    langs: {},
    themes: {},
  }
}

const shikiDir = path.join(__dirname, '../node_modules/shiki/dist')
const langFiles = fs
  .readdirSync(path.join(shikiDir, 'langs'))
  .filter(file => file.endsWith('.mjs'))
  .map(file => ({
    input: `shiki/dist/langs/${file}`,
    output: `vendor/shiki/langs/${file.replace('.mjs', '.js')}`,
    name: file.replace('.mjs', ''),
  }))

const themeFiles = fs
  .readdirSync(path.join(shikiDir, 'themes'))
  .filter(file => file.endsWith('.mjs'))
  .map(file => ({
    input: `shiki/dist/themes/${file}`,
    output: `vendor/shiki/themes/${file.replace('.mjs', '.js')}`,
    name: file.replace('.mjs', ''),
  }))

const coreModules = [
  {
    input: 'shiki/dist/core.mjs',
    output: 'vendor/shiki/core/core.js',
    name: 'core',
    type: 'typeof import("shiki/core")',
  },
  {
    input: 'shiki/dist/engine-javascript.mjs',
    output: 'vendor/shiki/core/engine-javascript.js',
    name: 'engine_javascript', // Use underscore in name to prevent TypeScript errors
    type: 'typeof import("shiki/engine/javascript")',
  },
  ...langFiles,
  ...themeFiles,
]

// Initialize manifest
const manifest = createManifest()

// Ensure vendor directory exists
if (!fs.existsSync(VENDOR_DIR)) {
  fs.mkdirSync(VENDOR_DIR, { recursive: true })
}

// Create necessary subdirectories
;['core', 'langs', 'themes'].forEach((dir) => {
  const fullPath = path.join(VENDOR_DIR, dir)
  if (!fs.existsSync(fullPath)) {
    fs.mkdirSync(fullPath, { recursive: true })
  }
})

// Build all modules and update manifest
for (const module of coreModules) {
  const inputPath = path.join(__dirname, '../node_modules', module.input)
  const outputPath = path.join(__dirname, '..', module.output)

  const dir = path.dirname(outputPath)
  if (!fs.existsSync(dir)) {
    fs.mkdirSync(dir, { recursive: true })
  }

  await buildModule(inputPath, outputPath, module.type || 'any')
  console.log(`Built ${path.basename(module.input)} -> ${path.basename(module.output)}`)

  // Update manifest
  if (module.input.includes('/langs/')) {
    manifest.langs[module.name] = {
      file: path.relative(VENDOR_DIR, outputPath),
      built: new Date().toISOString(),
    }
  }
  else if (module.input.includes('/themes/')) {
    manifest.themes[module.name] = {
      file: path.relative(VENDOR_DIR, outputPath),
      built: new Date().toISOString(),
    }
  }
  else {
    manifest.core.push({
      name: module.name,
      file: path.relative(VENDOR_DIR, outputPath),
      built: new Date().toISOString(),
    })
  }
}

// Write manifest
fs.writeFileSync(path.join(VENDOR_DIR, 'manifest.json'), JSON.stringify(manifest, null, 2))
